##
# @mainpage Phybers
# @brief Phybers is a library for Python that integrates multiple tractography and neural-fibers related algorithms. It includes the following sub-packages:
#
# \tableofcontents
# @subsection Modules Modules:
# <ol>
# <li>\ref phybers.clustering.clustering "Clustering" </li>
# <li>\ref phybers.segment.segment "Segment"</li>
# <li>\ref phybers.utils.utilities "Utilities"</li>
# <li>\ref phybers.fibervis "Fibervis"</li>
# </ol>
# Here goes a footer of the package or something
