"""
Phybers is a library for Python that integrates multiple tractography and neural-fibers related algorithms. It includes the following sub-packages:
    phybers.clustering "Clustering"
    phybers.segment "Segment"
    phybers.utils "Utilities"
    phybers.fibervis "Fibervis"
The functions of this package work mostly with file paths, both for input and output.
"""

__all__ = ['clustering', 'fibervis', 'segment', 'utils']